// Nolan's "Latin" drum machine

//C       C     G      D      G       Bb      C       Eb      F      Bb       C      G
[32.70, 65.41, 98, 146.83, 196.00, 233.08, 261.63, 311.13, 349.23, 466.16, 523.25, 783.99] @=> float Cm[];

//F       F       C       Ab     Eb      F       C       Eb       F
[43.65, 87.31, 130.81, 207.65, 311.13, 349.23, 523.23, 622.25, 698.46, 0, 0, 0] @=> float Fm[];

//Bb      Bb     F       Ab      F      Bb      D        Ab     Bb      D        F
[29.14, 58.27, 87.31, 103.83, 174.61, 233.08, 293.66, 415.30, 466.16, 587.33, 698.46, 0]@=> float Bbd[];

//D       D       C       F       D      F       Ab      C       F
[36.71, 73.42, 130.81, 174.61, 293.66, 349.23, 415.30, 523.25, 698.46, 0, 0, 0]@=> float Dhd[];
//G       G       D       G      B      D#       F       G        Bb      F
[49.00, 98.00, 146.83, 196.00, 246.94, 311.13, 349.23, 392.00, 466.16, 698.46, 0 ,0]@=> float Galt[];

//Eb      Eb      Bb      F      Bb      Eb     Gb       Ab      Eb
[38.89, 77.78, 116.54, 174.61, 233.08, 311.13, 369.99, 415.30, 622.25, 0, 0]@=> float Ebm[];

//Ab      Ab     Eb     Gb     C       Ab,    Bb       Eb      Gb      Ab      Eb
[25.96, 51.91, 77.78, 92.50, 130.81, 207.65, 233.08, 311.13, 369.99, 415.30, 622.25, 0]@=> float Abd[];

//Db      Db      Ab     C        F       Ab       Db      Ab     Db      Eb
[34.65, 69.30, 103.83, 130.81, 174.61, 207.65, 277.18, 415.30, 554.37, 622.25, 0, 0]@=> float DbM[];


[Cm, Fm, Bbd, Dhd, Galt, Cm, Ebm, Abd, DbM, Dhd, Galt, Cm] @=> float chords[][];

BlitSaw s[Cm.size()];
Gain g[Cm.size()];
NRev r;
LPF lpf[Cm.size()];
Pan2 p[Cm.size()];

.01 => r.mix;
0 => int chordCounter;
for(0 => int dead; dead < Cm.size(); dead++){
    s[dead] => g[dead] => lpf[dead] => r => p[dead] => dac;
    0 => p[dead].pan; 
    5000 => lpf[dead].freq;
    0 => s[dead].freq;
    .006 => g[dead].gain;
}

fun void progress(float chord[]){
    for(0 => int note; note < chord.size(); note++){
        chord[note] => s[note].freq;        
    }
}

fun void panparty(){
    for(0 => int party; party < 10; party++){
        for(0 => int w; w < p.size(); w++){
            Math.random2f(-.9, .9) => p[w].pan;
        }
        50::ms => now;
    }
}

fun void sweepy(int which){
    if(which == 0){
        Math.random2(100,5000) => int target;
        lpf[0].freq() => float currFreq;
        if(currFreq >= target){
            while(currFreq > target){
                currFreq - 10 => currFreq;
                for(0 => int f; f < lpf.size(); f++){
                    currFreq => lpf[f].freq;
                }
                10::ms => now;
            }
        }
        else if(currFreq < target){
            while(currFreq < target){
                currFreq + 10 => currFreq;
                for(0 => int f; f < lpf.size(); f++){
                    currFreq => lpf[f].freq;
                }
                10::ms => now;
            }
        }
    }
    if(which == 1){
        Math.random2f(0.1, 2.5) => float targetQ;
        lpf[0].Q() => float currQ;
        if(currQ >= targetQ){
            while(currQ > targetQ){
                currQ - .01 => currQ;
                for(0 => int f; f < lpf.size(); f++){
                    currQ => lpf[f].Q;
                }
                10::ms => now;
            }
        }
        else if(currQ < targetQ){
            while(currQ < targetQ){
                currQ + .01 => currQ;
                for(0 => int f; f < lpf.size(); f++){
                    currQ => lpf[f].Q;
                }
                10::ms => now;
            }
        }
    }
}
    
// quarter note duration
600.0::ms => dur Q;

// measures the measure number and 
class MachEvent extends Event
{
    // measure counter
    0 => int measure;
    // where we are in the cycle
    0 => int n;
    // max (free free to change this up, but make sure to update code that assume 16)
    16 => int max;
    // this can be set to tell shreds to stop
    1 => int keepGoing;
    
    // advance
    fun void next()
    { n++; if( n >= max ) { 0 => n; measure++; } }
}
// the event
MachEvent boom;

// reverb
NRev rL => dac.left;
NRev rR => dac.right;
.065 => rL.mix => rR.mix;    
// sound bank
SndBuf bank[10];
// Pan bank
Pan2 pan[bank.size()];
// snares
// (0 is no sound)
me.dir() + "samples/clave1.wav" => bank[1].read; 0 => pan[1].pan;
me.dir() + "samples/kick.wav" => bank[2].read; 0 => pan[2].pan;
me.dir() + "samples/shake.wav" => bank[3].read; 0 => pan[3].pan;
me.dir() + "samples/timbale.wav" => bank[4].read; 1 => pan[4].pan;


// connect
for( int i; i < bank.size(); i++ )
{
    // not sound
    0 => bank[i].gain;
    // connect
    bank[i] => pan[i];
    pan[i].left => rL;
    pan[i].right => rR;
}


// play: sound, velocity
fun void play( int n, float v )
{
    play( n, v, Math.random2f(1,1 ) );
}

// play: sound, velocity, rate
fun void play( int n, float v, float r )
{
    if( n < 0 || n >= bank.size() )
    {
        <<< "WARNING: out of bound BANK INDEX:", n >>>;
        return;
    }
    
    play( bank[n], v, r, 0::second );
}

// play: sound, velocity, rate, duration to wait
fun void play( SndBuf @ snd, float velocity, float rate, dur T )
{
    // start at beginning
    0 => snd.pos;
    // set velocity
    velocity => snd.gain;
    // set rate 
    rate => snd.rate;
    // wait
    if( T > 0::second ) T => now;
}

class Pattern
{
    1 => float volume;
    float probs[16];
    // NOTE: could be more arrays here for other data
}

class DrumPattern extends Pattern
{
    //picks SndBuf from array
    int sound;
    
    fun void run( int N )
    {
        int n;
        // go for N time
        while( N != 0 && boom.keepGoing )
        {
            // decrement
            if( N > 0 ) N--;
            // wait on the boom
            boom => now;
            // get the place in the measure
            boom.n => n;
            // logic
            if( probs[n] > 0 )
            {
                play( sound, probs[n] * volume );
            }
        }
    }
}

//shaker patterns
[[0.0,0.0,0.0,0.0,
   0.0, 0.0, 0.0, 0.0,
   0.0, 0.0, 0.0, 0.0,
   0.0, 0.0, 0.0, 0.0],
 [1.0, 1.0, 0, 0, 
  1.0, 0, 0, 0, 
  1.0, 0, 0, 1.0,
  0, 0, 1.0, 1.0],
 [1.0, 1.0, 1, 1,
  1, 1, 1, 1,
  1, 1, 1, 1,
  1, 1, 1, 1] ] @=> float shakers[][];

  
  
//claves
[ [0.0,0.0,0.0,0.0,
   0.0, 0.0, 0.0, 0.0,
   0.0, 0.0, 0.0, 0.0,
   0.0, 0.0, 0.0, 0.0],
  [ 1.0, 0.0, 0, 1.0,
  0, 0, 1.0, 0,
  0, 0, 1.0, 0,
  1.0, 0, 0, 0 ], //son32
  [ 0.0, 0.0, 1.0, 0.0,
  1.0, 0, 0, 0,
  1.0, 0, 0, 1.0,
  0, 0, 1.0, 0 ], //son23
  [ 1.0, 0.0, 0, 1.0,
  0, 0, 0, 1.0,
  0, 0, 1.0, 0,
  1.0, 0, 0, 0 ], //rumba32
  [ 0.0, 0.0, 1.0, 0.0,
  1.0, 0, 0, 0,
  1.0, 0, 0, 1.0, //rumba23
  0, 0, 0, 1.0 ]] @=> float claves[][];
  
// current kick
DrumPattern kick1;
0 => kick1.volume; // volume of the pattern (use to toggle/ramp patterns)
2 => kick1.sound; // which sound to use (try 5, 6, 7, or ??)
[ 1.0, 0, 0, 1,
  1, 0, 0, 1,
  1, 0, 0, .5,
  0, 1, 0, .5 ] @=> kick1.probs; // used as velocity but also can be probability
  
DrumPattern myshaker;
.85 => myshaker.volume; // volume on the pattern (use this to toggle/ramp patterns)
3 => myshaker.sound; // which sound (try others, like uh 1, 2, 3, or even 17)
shakers[0] @=> myshaker.probs;
  
DrumPattern myclave;
.85 => myclave.volume;
1 => myclave.sound;
claves[0] @=> myclave.probs;

DrumPattern timbale;
0 => timbale.volume;
4 => timbale.sound;
[ 0.0, 0, 0, 1.0,
  0, 0, 0, 0,
  0, 0, 0, 1.0,
  0, 0, 1.0, 0 ] @=> timbale.probs; //mutable


  
  
spork ~ myclave.run( -1 );
spork ~ kick1.run( -1 );
spork ~ myshaker.run( -1 );
spork ~ timbale.run( -1 );

KBHit kb;
0 => int claveSelector;
0 => int shakerSelector;
0 => int jInd;
0 => int lInd;
0 => int kInd;
0 => int iInd;
0 => int nInd;

while(true){
    kb.getchar() => int input;
    if(input != 0){
        <<<input>>>;
    }
    if(input == 'p'){
        1 +=> claveSelector;
    }
    if(input == '-'){
        1 -=> claveSelector;
    }
    if(input == 'h'){
        0 => claveSelector;
        0 => timbale.volume;
    }
    if(input == 75){//right
        if(pan[1].pan() > -1){
            pan[1].pan() - .1 => pan[1].pan; //pan clave
        }
    }
    if(input == 77){//left
        if(pan[1].pan() < 1){
            pan[1].pan() + .1 => pan[1].pan; //pan clave
        }
    }
    if(input == 72){//up
        if(kick1.volume < .8){
            kick1.volume + .1 => kick1.volume;
        }
    }
    if(input == 80){//down
        if(kick1.volume > .1){
            kick1.volume - .1 => kick1.volume;
        }
    }
    if(input == 'a'){
        1 +=> shakerSelector;
    }
    
    if(input == 'b'){
        1 -=> shakerSelector;
    }
    if(input == '1'){
        .75 => pan[4].pan;
        .85 => timbale.volume;
        Math.random2f(0,1) => float i; //add timbale
        Math.random2(0, 15) => int j;
        i => timbale.probs[j];
        if(jInd == 1){
            0 => jInd;
            1 => nInd;
        }
    }
    if(input == '2'){
        -1 * pan[4].pan() => pan[4].pan;
        Math.random2(0, 15) => int k; //subtract timbale; switch pan
        0 => timbale.probs[k];
        if(lInd == 1){
            0 => lInd;
            spork ~ panparty();
        }
    }
    if(input == 'j'){
        1 => jInd;
    }
    if(input == 'l'){
        1 => lInd;
    }
    if(input == 'i'){
        1 => iInd;
    }
    if(input == 'k'){
        1 => kInd;
    }

    if(input == '0' && Q < 700::ms){
        Q + 10::ms => Q;
    }
    if(input == '8' && Q > 11::ms){
        Q - 10::ms => Q;
    }
    if(input == '7'){
        if(iInd == 1){
            spork ~ sweepy(0);
        }
    }
    if(input == '9'){
        if(kInd == 1){
           spork ~ sweepy(1);
        }
    }
    if(input == 'c'){
        for(0 => int master; master < s.size(); master++){
            g[master].gain() - .001 => g[master].gain;
        }
    }
    if(input == 'z'){
        for(0 => int master; master < s.size(); master++){
            g[master].gain() + .001 => g[master].gain;
        }
    }
    
    // next boom, wake up all shreds waiting on boom event
    boom.broadcast();
    // wait for 16th
    Q/4 => now;
    if(boom.n == 15){
        claves[claveSelector%5] @=> myclave.probs;
        shakers[shakerSelector%3] @=> myshaker.probs;
        if(nInd == 1){
            0 => nInd;
            progress(chords[chordCounter%chords.size()]);
            chordCounter++;            
        }
    }

    <<<boom.n>>>;
    boom.next();
    
}
